package cz.muni.fi.pb162.iterator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * Iterátory sú určené na sekvenčné prechádzanie kolekcii
 * Fungujú pre všetky triedy implementujúce rozhranie Collection
 */
public class Demo {
    public static void main(String[] args) {

        // viac informáci o vytváraní nového zoznamu nájdete v balíčku list v triede Demo
        List<String> numbers = new ArrayList<>(List.of("One", "Two", "Three"));

        // na získanie iterátoru použijeme metódu iterator nad akoukoľvek kolekciou
        Iterator<String> iterator = numbers.iterator();

        // metóda hasNext() vráti true ak sa iterátor nenachádza na poslednom prvku kolekcie
        System.out.println("Má iterátor nasledujúci prvok: " + iterator.hasNext());

        // na získanie nasledujúceho prvku použijeme metódu next()
        System.out.println("Nasledujúci prvok je: " + iterator.next());

        // ak chceme odmazať prvok, ktorý bol naposledy získaný metódou next(), použijeme metódu remove()
        System.out.println("Odmazávam prvok \"One\" z kolekcie numbers " + numbers + " pomocou iterátoru");
        iterator.remove();
        System.out.println("numbers: " + numbers);

        // metóda remove môže byť zavolaná iba raz medzi volaniami metódy next()
        // po zavolaní metódy remove() iterátor stále odkazuje na prvok, ktorý sa už v kolekcii nenachádza
        // opätovné volanie funckie remove() by vyhodilo výnimku
        // iterator.remove(); // po odkomentovaní program padne za behu

        // na postupné prechádzanie kolekcie môžeme využiť for-cyklus alebo while-cyklus
        for (Iterator<String> iter = numbers.iterator(); iter.hasNext();) {
            // code ..
            break;
        }
        Set<String> ss = null; // doplnit třeba new HashSet<>()

        for(String n: ss) { // foreach nad Collection
            System.out.println(n);
        }

        while (iterator.hasNext()) {
            // code ..
            break;
        }

        // ak chceme volať rovnakú metódu nad každým prvkom môžeme využiť metódu forEachRemaining()
        // táto funkcia prijíma ako parameter funkciu
        // posúvanie funkcii ako parameter je nad rámec predmetu pb162
        System.out.println("Vypisujem zvyšné prvky pomocou metódy forEachRemaining()");
        iterator.forEachRemaining(System.out::println);
    }
}
