package cz.muni.fi.pb162.list;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * List je rozhranie implementujúce rozhranie Collection
 * Je veľmi podobné poliam, avšak jeho dĺžku je možné meniť
 * Jednotlivé prvky majú svoj index (celé nezáporné číslo)
 * Najpoužívanejšími implementáciami tohto rozhrania sú triedy ArrayList a LinkedList
 *
 * V tomto deme si ukážeme základné operácie na zoznamoch
 */
public class Demo {

    public static void main(String[] args) {
        // v metóde listCreation nájdete spôsoby vytvorenia nového zoznamu
        listCreation();

        // v metóde methodExamples nájdete príklady operácii nad zoznamami
        methodExamples();
    }

    private static void listCreation() {
        // nový prázdny zoznam vytvoríme následovne
        // zoznam je typu string - je doň možné vložiť len objekty typu String
        List<String> strings = new ArrayList<>();

        // nie je možné vytvoriť kolekciu primitivného typu
        // je nutne použiť objektový ekvivalent daného primitívneho typu
        //List<int> ints = new ArrayList<>(); // po odkomentovaní kód nepôjde skompilovať
        List<Integer> integers = new ArrayList<>();

        String[] stringArray = {"One", "Two", "Three"};

        // ak chceme previesť pole na zoznam môžeme využiť metódu asList z knižnice Arrays
        // tá však vracia nemodifikovatelný zoznam
        List<String> immutableListFromArray = Arrays.asList(stringArray);
        // immutableListFromArray.add("Four"); // po odkomentovaní program padne za behu

        // ak chceme vytvoriť zoznam vymenovaním prvkom môžeme využiť metódu "of" z triedy List
        // zoznam, ktorý takto vznikne je nemodifikovateľný
        List<String> listFromEnumeration = List.of("One", "Two", "Three");

        String s = listFromEnumeration.get(0);

        // ak chceme aby bol náš zoznam modifikovateľný musíme vytvoriť nový zoznam a prekopírovať odkazy na prvky z nemodifikovateľného zoznamu
        // využijeme na to konštruktor triedy ArrayList, ktorý príjma ako atribút inú kolekciu
        List<String> mutableListFromArray = new ArrayList<>(immutableListFromArray);
        System.out.println("Pridávam prvok \"Four\" do zoznamu mutableListFromArray.");
        mutableListFromArray.add("Four");
        System.out.println("mutableListFromArray.get(3) = " + mutableListFromArray.get(3));
    }

    private static void methodExamples() {
        List<String> numbers = new ArrayList<>();

        // na pridanie prvku na koniec zoznamu použijeme metódu add(newObject);
        System.out.println("Pridávam prvok \"Two\" do zoznamu numbers");
        numbers.add("Two");
        System.out.println("Zoznam numbers: " + numbers);

        // na pridanie prvku na konkrétny index použijeme metódu add(index, newObject);
        System.out.println("Pridávam prvok \"Zero\" do zoznamu numbers na index 0: " + numbers );
        numbers.add(0, "Zero");
        System.out.println("Zoznam numbers: " + numbers);

        // ak chceme nastaviť prvok na konkrétnom indexe, použijeme metódu set(index, newObject)
        System.out.println("Nastavujem prvok na indexe 1 na \"One\"");
        numbers.set(1, "One");
        System.out.println("Zoznam numbers: " + numbers);

        // ak chceme získať prvok na konkrétnom indexe, použijeme metódu get(index)
        System.out.println("Získavame prvok na indexe 0 zo zoznamu " + numbers);
        System.out.println("Prvok má hodnotu " + numbers.get(0));

        // ak chceme zistiť či list obsahuje prvok, využijeme metódu contains(element)
        System.out.println("Obsahuje zoznam nubers: " + numbers + " prvok \"Four\": " + numbers.contains("Four"));

        // ak chceme zistiť index prvku zo zoznamu, použijeme metódu indexOf(element)
        System.out.println("Index prvku \"Zero\" v zozname numbers: " + numbers + " je " + numbers.indexOf("Zero"));

        // na odstránenie prvku zo zoznamu, využijeme metódu remove(index)
        System.out.println("Odstraňujem prvok na indexe 0 zo zoznamu numbers: " + numbers);
        numbers.remove(0);
        System.out.println("Zoznam numbers: " + numbers);
    }
}
