package cz.muni.fi.pb162.set;

import java.util.*;

/**
 * Set je rozhranie implementujúce rozhranie Collection
 * Množina je podomná zoznamu, avšak všetky prvky v nej sú unikátne
 * Narozdiel od zoznamu, prvky množiny nie sú indexované
 * Najpouživanejšími implementáciami rozhrania Set sú triedy HashSet a TreeSet
 *
 * V tomto deme nájdete ukážky základnych operácii nad množinou
 */
public class Demo {
    public static void main(String[] args) {
        methodExamples();
    }

    private static void setCreation() {

        // novú prázdnu množinu vytvoríme následovne
        // množina je typu String - je do nej možné vložiť len objekty typu String
        Set<String> strings = new HashSet<>();

        // nie je možné vytvoriť kolekciu primitivného typu
        // je nutné použiť objektový ekvivalent daného primitívneho typu
        // Set<int> ints = new HashSet<>(); // po odkomentovaní kód nepôjde skompilovať
        Set<Integer> integers = new HashSet<>();

        String[] stringArray = {"One", "Two", "Three"};

        // v triede Arrays sa metóda, ktorá by pole previedla priamo na množinu, nenachádza
        // vieme však previesť pole na zoznam a ten vložiť do konštruktoru novej množiny
        Set<String> setFromArrayViaList = new HashSet<>(Arrays.asList(stringArray));

        // druhým spôsobom je využiť metódu "addAll" z knižnice Collections
        // tá prekopíruje všetky prvky z poľa do našej množiny
        // ako prvé je však nutné vytvoriť novú alebo použiť existujúcu množinu
        Set<String> setFromArrayViaCollectionsLibrary = new HashSet<>();
        Collections.addAll(setFromArrayViaCollectionsLibrary, stringArray);

        // ak chceme vytvoriť novú množinu vymenovaním prvkov môžeme využiť metódu "of" z triedy Set
        // množina, ktorá takto vznikne je nemodifikovateľná
        Set<String> immutableEnumeratedSet = Set.of("One", "Two", "Three");

        // ak chceme aby bola naša množina modifikovateľná musíme vytvoriť novú a prekopírovať odkazy na prvky z nemodifikovateľnéj množiny
        // využijeme na to konštruktor triedy HashSet (poprípade TreeSet), ktorý príjma parameter typu Collection
        Set<String> mutableSetFromImmutable = new HashSet<>(immutableEnumeratedSet);
    }

    private static void methodExamples() {
        Set<String> numbers = new HashSet<>();

        // na pridanie nového prvku do množiny použijeme metódu add
        System.out.println("Pridávam prvok \"One\" do množiny numbers");
        numbers.add("One");
        System.out.println("Množina numbers: " + numbers);

        // ak pridáme znovu prvok, ktorý sa už v množine nachádza nevznikne duplikát
        System.out.println("Znovu pridávam prvok \"One\" do množiny numbers");
        numbers.add("One");
        System.out.println("Množina numbers: " + numbers);

        // ak chceme overiť či množina obsahuje prvok využijeme metódu contains
        System.out.println("Obsahuje množina numbers prvok \"One\": " + numbers.contains("One"));
        System.out.println("Množina numbers: " + numbers);

        // keďže množina nie je indexovaná nie je možné z nej získať prvky pomocou indexu ako v zozname
        // numbers.get(0); // po odkomentovaní nepôjde skompilovať

        // ak chceme odstrániť nejaký prvok využijeme metódu remove
        System.out.println("Odstraňujem prvok \"One\" z množiny numbers");
        numbers.remove("One");
        System.out.println("Množina numbers: " + numbers);

        // trieda TreeSet predstavuje usporiadanú množinu
        Set<String> words = new TreeSet<>();
        words.add("Car");

        System.out.println("Pridávam prvok \"Avocado\" do usporiadanej množiny words");
        words.add("Avocado");
        System.out.println("Množina words: " + words);

        System.out.println("Pridávam prvok \"Spaceship\" do usporiadanej množiny words");
        words.add("Spaceship");
        System.out.println("Množina words: " + words);

        // ostatné metódy fungujú rovnako ako nad triedou HashSet
    }
}
